@@ -45,33 +45,49 @@ enum {
         STOP_KEY,
         PREVIOUS_KEY,
         NEXT_KEY,
+        REWIND_KEY,
+        FORWARD_KEY,
+        REPEAT_KEY,
+        RANDOM_KEY,
+        MAGNIFIER_KEY,
+        SCREENREADER_KEY,
+        ON_SCREEN_KEYBOARD_KEY,
         HANDLED_KEYS
 };
 
 static struct {
         int key_type;
         const char *settings_key;
+        const char *hard_coded;
         Key *key;
 } keys[HANDLED_KEYS] = {
-        { TOUCHPAD_KEY, "touchpad", NULL },
-        { MUTE_KEY, "volume-mute",NULL },
-        { VOLUME_DOWN_KEY, "volume-down", NULL },
-        { VOLUME_UP_KEY, "volume-up", NULL },
-        { POWER_KEY, "power", NULL },
-        { EJECT_KEY, "eject", NULL },
-        { HOME_KEY, "home", NULL },
-        { MEDIA_KEY, "media", NULL },
-        { CALCULATOR_KEY, "calculator", NULL },
-        { SEARCH_KEY, "search", NULL },
-        { EMAIL_KEY, "email", NULL },
-        { SCREENSAVER_KEY, "screensaver", NULL },
-        { HELP_KEY, "help", NULL },
-        { WWW_KEY, "www", NULL },
-        { PLAY_KEY, "play", NULL },
-        { PAUSE_KEY, "pause", NULL },
-        { STOP_KEY, "stop", NULL },
-        { PREVIOUS_KEY, "previous", NULL },
-        { NEXT_KEY, "next", NULL },
+        { TOUCHPAD_KEY, "touchpad", NULL, NULL },
+        { MUTE_KEY, "volume-mute", NULL, NULL },
+        { VOLUME_DOWN_KEY, "volume-down", NULL, NULL },
+        { VOLUME_UP_KEY, "volume-up", NULL, NULL },
+        { POWER_KEY, "power", NULL, NULL },
+        { EJECT_KEY, "eject", NULL, NULL },
+        { HOME_KEY, "home", NULL, NULL },
+        { MEDIA_KEY, "media", NULL, NULL },
+        { CALCULATOR_KEY, "calculator", NULL, NULL },
+        { SEARCH_KEY, "search", NULL, NULL },
+        { EMAIL_KEY, "email", NULL, NULL },
+        { SCREENSAVER_KEY, "screensaver", NULL, NULL },
+        { HELP_KEY, "help", NULL, NULL },
+        { WWW_KEY, "www", NULL, NULL },
+        { PLAY_KEY, "play", NULL, NULL },
+        { PAUSE_KEY, "pause", NULL, NULL },
+        { STOP_KEY, "stop", NULL, NULL },
+        { PREVIOUS_KEY, "previous", NULL, NULL },
+        { NEXT_KEY, "next", NULL, NULL },
+        /* Those are not configurable in the UI */
+        { REWIND_KEY, NULL, "XF86AudioRewind", NULL },
+        { FORWARD_KEY, NULL, "XF86AudioForward", NULL },
+        { REPEAT_KEY, NULL, "XF86AudioRepeat", NULL },
+        { RANDOM_KEY, NULL, "XF86AudioRandomPlay", NULL},
+        { MAGNIFIER_KEY, "magnifier", NULL, NULL },
+        { SCREENREADER_KEY, "screenreader", NULL, NULL },
+        { ON_SCREEN_KEYBOARD_KEY, "on-screen-keyboard", NULL, NULL },
 };
 
 #endif /* __ACME_H__ */
